## 1.文件说明
- rtsi/rtsi.py 和 rtsi/serialize.py 是 rtsi 库文件  
- sample 是rtsi的示例，包括了 ：
  - rtsi_simple_sample.py ： 较为全面地简单地展示了 RTSI 协议的功能。指令：`python rtsi_simple_sample.py --host example_IP`
  - rtsi_record_sample.py ： 订阅 output 并记录于 .csv 文件中。指令： `python rtsi_record_sample.py --host example_IP --sample 100` （记录100组数据）
  - rtsi_multiple_sample.py ： RTSI 多配方订阅示例。 指令：`python rtsi_multiple_sample.py --host example_IP`
  - rtsi_input_sample.py ： RTSI input 订阅示例。指令 ： `python rtsi_input_sample.py --host example_IP`

## 2.rtsi类及方法说明
- rtsi.py 定义了 rtsi 类，说明
    - 导入： from rtsi import rtsi
    - 创建类：con = rtsi(hostname, port = 30004)
        - hostname：机器人ip地址
        - port：机器人 rtsi 端口，缺省值为 30004（控制器目前仅支持30004端口为rtsi端口）

    - rtsi.connect()：socket链接机器人rtsi服务

    - rtsi.disconnect()：断开链接

    - rtsi.version_check()：rtsi协议检查

    - rtsi.controller_version()：获取控制器版本
        - 返回值：列表：\[主版本, 次版本, bugfix版本, 构建版本\]

    - rtsi.input_subscribe(variables)：订阅输入变量
        - variables：字符串，订阅变量的名，变量名之间用','隔开，例如："speed_slider_mask,speed_slider_fraction"
        - 返回值：将会返回一个 DataObject 对象，该对象在serialize.py里定义，在 rtsi.set_input() 用到
        - 使用例子：对于返回的 DataObject 对象，可以直接这样使用来设置订阅的变量：  
            > ...(完成前置步骤)  
            input_data = instance.input_subscribe("speed_slider_mask,speed_slider_fraction")  #订阅speed_slider_mask,speed_slider_fraction两个变量  
            input_data.speed_slider_mask = 1  #设置 speed_slider_mask 为 1  
            input_data.speed_slider_fraction = 1  #设置 speed_slider_fraction 为 1  
            instance.set_input(input_data) #（详细见后文）把设置好的变量发送给机器人rtsi服务，这样就能看到机器人速度被调整为100%（因为订阅的这两个变量是用于调整速度的）

    - rtsi.output_subscribe(variables, frequency = 125)：订阅输出变量
        - variables：字符串，订阅变量的名，变量名之间用','隔开，例如："payload,actual_q"
        - frequency：输出的频率
        - 返回值： DataConfig 对象，该对象含属性：id（配方ID）、names（订阅变量的list）、types（订阅变量类型的list）

    - rtsi.start()：向控制器rtsi服务发送开始信号。发送该信号之后就可以接收订阅的输出变量，或者设置订阅的输入变量

    - rtsi.pause()：向控制器rtsi服务发送暂停信号。发送该信号之后rtsi的输出变量就暂停发送，输入变量就不可设置。

    - rtsi.set_input(input_data)：设置订阅的输入变量
        - input_data：为 DataObject 对象，该对象可以直接 实例.变量名 来操作变量的值，例如：instance.speed_slider_fraction = 1
        - 使用例子：
            >...(完成前置步骤)  
            input_data = instance.input_subscribe("speed_slider_mask,speed_slider_fraction")  #订阅speed_slider_mask,speed_slider_fraction两个变量  
            input_data.speed_slider_mask = 1  #设置 speed_slider_mask 为 1  
            input_data.speed_slider_fraction = 1  #设置 speed_slider_fraction 为 1  
            instance.set_input(input_data) #把设置好的变量发送给机器人rtsi服务，这样就能看到机器人速度被调整为100%（因为订阅的这两个变量是用于调整速度的）
        
    - rtsi.send_message(message, source = b"Python Client", type = serialize.Message.INFO_MESSAGE)：向机器人发送消息，该消息会显示在界面的日志上
        - message：消息内容
        - source：消息源
        - type：消息类型
    
    - rtsi.get_output_data_buffered(buffer_limit = None)：接收缓存里的所有输出数据，当缓存中存在不止一个数据包时，将会接收他们
      - buffer_limit：缓存的大小
    
    - rtsi.get_output_data() : 接收最后一个输出数据