import sys
sys.path.append('..')
from rtsi import serialize
import xml.etree.ElementTree as ET
import csv

class record():
    def __init__(self, filename) -> None:
        self.__filename = filename
        tree = ET.parse(self.__filename)
        root = tree.getroot()
        recipes = [self.__parse(r) for r in root.findall('recipe')]
        self.__dictionary = dict()
        for r in recipes:
            self.__dictionary[r.key] = r

    def __parse(self, recipe_node):
        self.key = recipe_node.get('key')
        self.names = [f.get('name') for f in recipe_node.findall('field')]
        self.types = [f.get('type') for f in recipe_node.findall('field')]
        return self
    
    def get_recipe(self, key):
        r = self.__dictionary[key]
        return r.names, r.types

class CSVRecord():
    def __init__(self, filename, conf : record) -> None:
        self.filename = filename
        self.fp = open(self.filename, 'w')
        self.writer = csv.writer(self.fp)
        self.names , self.types = conf.get_recipe('out')
        self.nameheader = list()
        for i in range(len(self.types)):
            size = serialize.get_item_size(self.types[i])
            if size > 1:
                for j in range(size):
                    self.nameheader.append(self.names[i] + '_' + str(j))
            else:
                self.nameheader.append(self.names[i])

    def writeheader(self):
        self.writer.writerow(self.nameheader)
    
    def writerow(self, data):
        temp = list()
        for i in range(len(self.types)):
            size = serialize.get_item_size(self.types[i])
            if size > 1:
                temp.extend(data.__dict__[self.names[i]])
            else:
                temp.append(data.__dict__[self.names[i]])
        self.writer.writerow(temp)

            
